@@ -1,3 +1,9 @@
+/******************************************************************************
+* Copyright 1996-2013 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+
 import java.io.*;
 import java.text.*;
 import java.util.*;
@@ -88,6 +94,11 @@ public class ReadCDF implements CDFConstants{
               System.out.println("Compression:  "+cdf.getCompression());
               if (cdf.getCompressionType() != 0)
                 System.out.println("cPct:         "+cdf.getCompressionPct());
+              if (new Integer(cdf.getVersion().substring(0,1)).intValue() > 3) {
+                long lsLastUpdated = cdf.getLeapSecondLastUpdated();
+                if (lsLastUpdated > 0)
+		  System.out.println("LeapSecondLastUpdated:     "+lsLastUpdated);
+              }
               System.out.println("Cache Size:   "+cdf.confirmCDFCacheSize());
   
               /**********************************************/
@@ -134,17 +145,17 @@ public class ReadCDF implements CDFConstants{
                           Object data = entry.getData();
                           if (eDataType == CDF_EPOCH) {
                               CDFUtils.printData (
-                                Epoch.encode(((Double) data).doubleValue()));
+                                Epoch.toEncode(((Double) data).doubleValue()));
                           } else if (eDataType == CDF_EPOCH16) {
-                              CDFUtils.printData (Epoch16.encode((double[])data));
+                              CDFUtils.printData (Epoch16.toEncode((double[])data));
                           } else if (eDataType == CDF_TIME_TT2000) {
                               CDFUtils.printData (
-                                CDFTT2000.encode(((Long)data).longValue()));
+                                CDFTT2000.toEncode(((Long)data).longValue()));
                           } else {
-                            if (eDataType == CDF_CHAR || eDataType == CDF_UCHAR)
+                            if (CDFUtils.isStringDataType(eDataType))
                               System.out.print("\"");
                             CDFUtils.printData (data);
-                            if (eDataType == CDF_CHAR || eDataType == CDF_UCHAR)
+                            if (CDFUtils.isStringDataType(eDataType))
                               System.out.print("\"");
                           }
                           System.out.println (" ");
@@ -183,24 +194,36 @@ public class ReadCDF implements CDFConstants{
                        Variable v = cdf.getVariable(entry.getID());
                        System.out.print ("\t"+v.getName()+" ("+
                                         CDFUtils.getStringDataType(eDataType)+
-                                        "/"+entry.getNumElements()+
-                                        "): ");
-
+                                        "/"+entry.getNumElements()+"): ");
                        Object data = entry.getData();
                        if (eDataType == CDF_EPOCH) {
                            CDFUtils.printData (
-                               Epoch.encode(((Double) data).doubleValue()));
+                               Epoch.toEncode(((Double) data).doubleValue()));
                        } else if (eDataType == CDF_EPOCH16) {
-                           CDFUtils.printData (Epoch16.encode((double[])data));
+                           CDFUtils.printData (Epoch16.toEncode((double[])data));
                        } else if (eDataType == CDF_TIME_TT2000) {
                            CDFUtils.printData (
-                               CDFTT2000.encode(((Long) data).longValue()));
+                               CDFTT2000.toEncode(((Long) data).longValue()));
                        } else {
-                          if (eDataType == CDF_CHAR || eDataType == CDF_UCHAR)
-                            System.out.print("\"");
-                          CDFUtils.printData (data);
-                          if (eDataType == CDF_CHAR || eDataType == CDF_UCHAR)
-                            System.out.print("\"");
+                          if (entry.getScope() == VARIABLE_SCOPE &&
+                              CDFUtils.isStringDataType(eDataType)) {
+                            if (data instanceof String[]) {
+                              long ix = ((String[])data).length;
+                              System.out.println("\t("+ix+" string"+(ix>1?"s":"")+") ");
+                              int iy;
+                              for (iy=0; iy<ix; ++iy) {
+                                if (iy != (ix-1))
+                                  System.out.println("\t\t\t\t\""+((String[])data)[iy]+"\""); 
+                                else
+                                  System.out.print("\t\t\t\t\""+((String[])data)[iy]+"\"");
+                              }
+                            } else {
+                              System.out.print("\"");
+                              CDFUtils.printData (data);
+                              System.out.print("\"");
+                            }
+                          } else
+                            CDFUtils.printData (data);
                        }
                        System.out.println (" ");
                     }
@@ -220,7 +243,7 @@ public class ReadCDF implements CDFConstants{
               Vector  vars = cdf.getVariables();
 
               System.out.println("\nVariable Information ("+n+" variables)\n"+
-                                 "=========================================");
+                                 "===================================================");
               for (Enumeration e = vars.elements() ; e.hasMoreElements() ;) {
                 Variable v = (Variable) e.nextElement();
 
@@ -233,7 +256,7 @@ public class ReadCDF implements CDFConstants{
                 dt = v.getDataType();
                 dataType = CDFUtils.getStringDataType(dt);
                 dataType = dataType + "/" + String.valueOf(v.getNumElements());
-                noOfBlanks = 14 - dataType.length();
+                noOfBlanks = 20 - dataType.length();
                 for (i=0; i < noOfBlanks; i++)
                     dataType = dataType + " ";
 
@@ -243,7 +266,7 @@ public class ReadCDF implements CDFConstants{
                     if (i > 0) System.out.print (",");
                     System.out.print (dimSizes[i]); 
                 }
-                System.out.print ("]\t");
+                System.out.print ("]  ");
                 // if (numDims == 1) System.out.print ("\t");
 
                 System.out.print((v.getRecVariance() ? "T" : "F")+"/");
@@ -257,16 +280,20 @@ public class ReadCDF implements CDFConstants{
                      System.out.print (" sparseRecord=\""+sparseRecord+"\"");
 
                 // Print the user-define pad value if one exists. 
-		if (v.checkPadValueExistence()) {
+		// if (v.checkPadValueExistence()) {
                   padValue = v.getPadValue(); 
                   if (dt == CDF_EPOCH)
-                    padValue = Epoch.encode(((Double) padValue).doubleValue());
+                    padValue = Epoch.toEncode(((Double) padValue).doubleValue());
                   else if (dt == CDF_EPOCH16)
-                    padValue = Epoch16.encode((double[]) padValue);
+                    padValue = Epoch16.toEncode((double[]) padValue);
                   else if (dt == CDF_TIME_TT2000)
-                    padValue = CDFTT2000.encode(((Long) padValue).longValue());
-                  System.out.print (" PadValue=\""+padValue+"\"");
-                }
+                    padValue = CDFTT2000.toEncode(((Long) padValue).longValue());
+                  System.out.print (" PadValue="+((CDFUtils.isStringDataType(dt)||
+                                                   CDFUtils.isEpochDataType(dt))?
+                                                  "\"":"")+padValue+
+                                                  ((CDFUtils.isStringDataType(dt)||
+                                                   CDFUtils.isEpochDataType(dt))?"\"":""));
+                // }
 
                 blockingFactor = v.getBlockingFactor();
                 if (blockingFactor > 0)
@@ -318,14 +345,14 @@ public class ReadCDF implements CDFConstants{
                   if (pv != null) {
 		    if (myDT == CDF_EPOCH)
 		      System.out.println ("Pad value:        "+
-                         Epoch.encode(((Double)pv).doubleValue()));
+                         Epoch.toEncode(((Double)pv).doubleValue()));
 		    else if (myDT == CDF_EPOCH16)
 		      System.out.println ("Pad value:        "+
-                         Epoch16.encode((double[])pv));
+                         Epoch16.toEncode((double[])pv));
                     else if (myDT == CDF_TIME_TT2000)
                       System.out.println ("Pad value:        "+
                          CDFTT2000.toUTCstring(((Long)pv).longValue()));
-                    else if (myDT == CDF_CHAR || myDT == CDF_UCHAR)
+                    else if (CDFUtils.isStringDataType(myDT))
                       System.out.println ("Pad value:        \""+ pv+"\"");
                     else 
 		      System.out.println ("Pad value:        "+ pv);
